#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

import json
import logging
import os

from flask import Blueprint, jsonify, request, Response
from oceandb_driver_interface.search_model import FullTextModel, QueryModel
from ocean_lib.config_provider import ConfigProvider
from ocean_lib.web3_internal.contract_handler import ContractHandler
from ocean_lib.web3_internal.web3_provider import Web3Provider
from ocean_lib.ocean.util import get_web3_connection_provider
from ocean_lib.config import Config as OceanConfig
from plecos.plecos import (
    is_valid_dict_local,
    list_errors_dict_local,
)

from aquarius.app.auth_util import has_update_request_permission, get_signer_address
from aquarius.app.dao import Dao
from aquarius.app.util import (
    make_paginate_response,
    datetime_converter,
    get_metadata_from_services,
    sanitize_record,
    list_errors,
    get_request_data)
from aquarius.events.metadata_updater import MetadataUpdater
from aquarius.events.util import get_artifacts_path, get_network_name
from aquarius.log import setup_logging
from aquarius.myapp import app

ConfigProvider.set_config(OceanConfig(app.config['CONFIG_FILE']))
Web3Provider.init_web3(provider=get_web3_connection_provider(os.environ.get('EVENTS_RPC', '')))
ContractHandler.set_artifacts_path(get_artifacts_path())
if get_network_name().lower() == 'rinkeby':
    from web3.middleware import geth_poa_middleware
    Web3Provider.get_web3().middleware_stack.inject(geth_poa_middleware, layer=0)

setup_logging()
assets = Blueprint('assets', __name__)

# Prepare OceanDB
dao = Dao(config_file=app.config['CONFIG_FILE'])
logger = logging.getLogger('aquarius')


@assets.route('', methods=['GET'])
def get_assets_ids():
    """Get all asset IDs.
    ---
    tags:
      - ddo
    responses:
      200:
        description: successful action
    """
    asset_with_id = dao.get_all_listed_assets()
    asset_ids = [a['id'] for a in asset_with_id]
    return Response(json.dumps(asset_ids), 200, content_type='application/json')


@assets.route('/ddo/<did>', methods=['GET'])
def get_ddo(did):
    """Get DDO of a particular asset.
    ---
    tags:
      - ddo
    parameters:
      - name: did
        in: path
        description: DID of the asset.
        required: true
        type: string
    responses:
      200:
        description: successful operation
      404:
        description: This asset DID is not in OceanDB
    """
    try:
        asset_record = dao.get(did)
        return Response(sanitize_record(asset_record), 200, content_type='application/json')
    except Exception as e:
        logger.error(f'get_ddo: {str(e)}')
        return f'{did} asset DID is not in OceanDB', 404


@assets.route('/ddo', methods=['GET'])
def get_asset_ddos():
    """Get DDO of all assets.
    ---
    tags:
      - ddo
    responses:
      200:
        description: successful action
    """
    _assets = dao.get_all_listed_assets()
    for _record in _assets:
        sanitize_record(_record)
    return Response(json.dumps(_assets, default=datetime_converter), 200,
                    content_type='application/json')


@assets.route('/metadata/<did>', methods=['GET'])
def get_metadata(did):
    """Get metadata of a particular asset
    ---
    tags:
      - metadata
    parameters:
      - name: did
        in: path
        description: DID of the asset.
        required: true
        type: string
    responses:
      200:
        description: successful operation.
      404:
        description: This asset DID is not in OceanDB.
    """
    try:
        asset_record = dao.get(did)
        metadata = get_metadata_from_services(asset_record['service'])
        return Response(sanitize_record(metadata), 200, content_type='application/json')
    except Exception as e:
        logger.error(f'get_metadata: {str(e)}')
        return f'{did} asset DID is not in OceanDB', 404


###########################
# SEARCH
###########################

@assets.route('/ddo/query', methods=['GET'])
def query_text():
    """Get a list of DDOs that match with the given text.
    ---
    tags:
      - ddo
    parameters:
      - name: text
        in: query
        description: ID of the asset.
        required: true
        type: string
      - name: sort
        in: query
        type: object
        description: Key or list of keys to sort the result
        example: {"value":1}
      - name: offset
        in: query
        type: int
        description: Number of records per page
        example: 100
      - name: page
        in: query
        type: int
        description: Page showed
        example: 1
    responses:
      200:
        description: successful action
    """
    data = get_request_data(request)
    assert isinstance(
        data, dict), 'invalid `args` type, should already formatted into a dict.'
    sort = data.get('sort', None)
    if sort is not None and isinstance(sort, str):
        sort = json.loads(sort)

    search_model = FullTextModel(text=data.get('text', None),
                                 sort=sort,
                                 offset=int(data.get('offset', 100)),
                                 page=int(data.get('page', 1)))
    query_result = dao.query(search_model)
    for i in query_result[0]:
        sanitize_record(i)

    response = make_paginate_response(query_result, search_model)
    return Response(json.dumps(response, default=datetime_converter), 200,
                    content_type='application/json')


@assets.route('/ddo/query', methods=['POST'])
def query_ddo():
    """Get a list of DDOs that match with the executed query.
    ---
    tags:
      - ddo
    consumes:
      - application/json
    parameters:
      - in: body
        name: body
        required: true
        description: Asset metadata.
        schema:
          type: object
          properties:
            query:
              type: string
              description: Query to realize
              example: {"value":1}
            sort:
              type: object
              description: Key or list of keys to sort the result
              example: {"value":1}
            offset:
              type: int
              description: Number of records per page
              example: 100
            page:
              type: int
              description: Page showed
              example: 1
    responses:
      200:
        description: successful action
    """
    assert isinstance(request.json, dict), 'invalid payload format.'
    data = request.json
    assert isinstance(
        data, dict), 'invalid `body` type, should be formatted as a dict.'

    native_query = None
    if 'nativeSearch' in data:
        native_query = data.pop('nativeSearch')

    query = data.get('query', {})
    if not native_query and 'nativeSearch' in query:
        native_query = query.pop('nativeSearch')

    if native_query:
        query_result, search_model = process_es_query(data)
    else:
        search_model = QueryModel(query=query, sort=data.get('sort'),
                                  offset=data.get('offset', 100),
                                  page=data.get('page', 1))
        query_result = dao.query(search_model)

    for ddo in query_result[0]:
        sanitize_record(ddo)

    response = make_paginate_response(query_result, search_model)
    return Response(json.dumps(response, default=datetime_converter), 200,
                    content_type='application/json')


@assets.route('/ddo/esquery', methods=['POST'])
def es_query_ddo():
    """Get a list of DDOs that match with the executed query.
    ---
    tags:
      - ddo
    consumes:
      - application/json
    parameters:
      - in: body
        name: body
        required: true
        description: Asset metadata.
        schema:
          type: object
          properties:
            query:
              type: string
              description: Query to realize
              example: {"value":1}
            text:
              type: string or list of strings
              description: Fulltext query
              example: ["text to search"]
            sort:
              type: object
              description: Key or list of keys to sort the result
              example: {"value":1}
            offset:
              type: int
              description: Number of records per page
              example: 100
            page:
              type: int
              description: Page showed
              example: 1
    responses:
      200:
        description: successful action

    example:
        {"query": {"query_string": {"query": "(covid) -isInPurgatory:true"}}, "offset":1, "page": 1}

    """
    assert isinstance(request.json, dict), 'invalid payload format.'
    data = request.json
    query_result, search_model = process_es_query(data)
    for ddo in query_result[0]:
        sanitize_record(ddo)

    response = make_paginate_response(query_result, search_model)
    return Response(json.dumps(response, default=datetime_converter), 200,
                    content_type='application/json')


def process_es_query(data):
    text = data.get('text', [])
    query = data.get('query')
    if not text and 'text' in query:
        text = query.pop('text')

    if isinstance(text, str):
        text = [text]
    temp = []
    for v in text:
        temp.append(v.replace(':', '\\'))
    querystr = json.dumps(query)
    did_str = 'did:op:'
    esc_did_str = 'did\\\:op\\\:'
    querystr = querystr.replace(esc_did_str, did_str)
    query = json.loads(querystr.replace(did_str, esc_did_str))
    text = temp

    sort = data.get('sort')
    page = data.get('page', 1)
    offset = data.get('offset', 100)

    query_result = dao.run_es_query(query, sort, page, offset, text)
    return query_result, FullTextModel('', sort, offset, page)


###########################
# VALIDATE
###########################

@assets.route('/ddo/validate', methods=['POST'])
def validate():
    """Validate metadata content.
    ---
    tags:
      - ddo
    consumes:
      - application/json
    parameters:
      - in: body
        name: body
        required: true
        description: Asset metadata.
        schema:
          type: object
    responses:
      200:
        description: successfully request.
      500:
        description: Error
    """
    assert isinstance(request.json, dict), 'invalid payload format.'
    data = request.json
    assert isinstance(
        data, dict), 'invalid `body` type, should be formatted as a dict.'

    if is_valid_dict_local(data):
        return jsonify(True)
    else:
        res = jsonify(list_errors(list_errors_dict_local, data))
        return res


@assets.route('/ddo/update/<did>', methods=['PUT'])
def update_ddo_info(did):
    assert request.json and isinstance(request.json, dict), 'invalid payload format.'
    data = request.json
    address = data.get('adminAddress', None)
    if not address or not has_update_request_permission(address):
        return jsonify(error=f'Unauthorized.'), 401

    _address = None
    signature = data.get('signature', None)
    if signature:
        _address = get_signer_address(address, signature, logger)

    if not _address or _address.lower() != address.lower():
        return jsonify(error=f'Unauthorized.'), 401

    try:
        asset_record = dao.get(did)
        if not asset_record:
            return jsonify(error=f'Asset {did} not found.'), 404

        updater = MetadataUpdater(oceandb=dao.oceandb, web3=Web3Provider.get_web3(), config=ConfigProvider.get_config())
        updater.do_single_update(asset_record)

        return jsonify('acknowledged.'), 200
    except Exception as e:
        logger.error(f'get_metadata: {str(e)}')
        return f'{did} asset DID is not in OceanDB', 404


@assets.route('/ddo/<did>', methods=['DELETE'])
def delist_ddo(did):
    assert request.json and isinstance(request.json, dict), 'invalid payload format.'
    data = request.json
    address = data.get('adminAddress', None)
    if not address or not has_update_request_permission(address):
        return jsonify(error=f'Unauthorized.'), 401

    _address = None
    signature = data.get('signature', None)
    if signature:
        _address = get_signer_address(address, signature, logger)

    if not _address or _address.lower() != address.lower():
        return jsonify(error=f'Unauthorized.'), 401

    try:
        asset_record = dao.get(did)
        if not asset_record:
            return jsonify(error=f'Asset {did} not found.'), 404

        updater = MetadataUpdater(oceandb=dao.oceandb, web3=Web3Provider.get_web3(), config=ConfigProvider.get_config())
        updater.do_single_update(asset_record)

        return jsonify('acknowledged.'), 200
    except Exception as e:
        logger.error(f'get_metadata: {str(e)}')
        return f'{did} asset DID is not in OceanDB', 404



import json
import os

from ocean_lib.web3_internal.web3helper import Web3Helper
from web3 import Web3


def get_signer_address(message, signature, logger):
    """
    Get signer address of a previous signed message
    :param str message: Message
    :param str signature: Signature obtain with web3.eth.personal.sign
    :param logger: logging object
    :return: Address or None in case of error
    """
    try:
        logger.debug(f'got "{message}" as a message')
        address_recovered = Web3Helper.personal_ec_recover(message, signature)
        logger.debug(f'got "{address_recovered}" as address_recovered')
        return address_recovered
    except Exception as e:
        logger.error(f'get_signer_address: {e}')
        return None


def sanitize_addresses(addresses):
    return [Web3.toChecksumAddress(a) for a in addresses if Web3.isAddress(a)]


def compare_eth_addresses(address, checker, logger):
    """
    Compare two addresses and return TRUE if there is a match
    :param str address: Address
    :param str checker: Address to compare with
    :param logger: instance of logging
    :return: boolean
    """
    logger.debug('compare_eth_addresses address: %s' % address)
    logger.debug('compare_eth_addresses checker: %s' % checker)
    if not Web3.isAddress(address):
        logger.debug("Address is not web3 valid")
        return False
    if not Web3.isAddress(checker):
        logger.debug("Checker is not web3 valid")
        return False
    return Web3.toChecksumAddress(address) == Web3.toChecksumAddress(checker)


def has_update_request_permission(address):
    vip_list = os.getenv('AQUA_VIP_ACCOUNTS', '[]')
    try:
        vip_list = json.loads(vip_list)
        return isinstance(vip_list, list) and bool(address in vip_list)
    except Exception:
        return False



import json
import logging
import os

from flask import Blueprint, request, Response

from ocean_lib.config_provider import ConfigProvider
from ocean_lib.models.bpool import BPool
from ocean_lib.ocean.ocean import Ocean
from ocean_lib.ocean.util import from_base_18

from aquarius.app.pool_helper import build_liquidity_and_price_history
from aquarius.app.util import get_request_data

pools = Blueprint('pools', __name__)

logger = logging.getLogger(__name__)


@pools.route('/history/<poolAddress>', methods=['GET'])
def get_liquidity_history(poolAddress):
    """

    :param poolAddress:
    :return: json object with two keys: `ocean` and `datatoken`
      each has a list of datapoints sampled at specific time intervals from the pools liquidity history.
    """
    try:
        result = dict()
        ocean = Ocean(ConfigProvider.get_config())
        pool = BPool(poolAddress)
        dt_address = ocean.pool.get_token_address(poolAddress, pool, validate=False)
        swap_fee = from_base_18(pool.getSwapFee())
        ocn_weight = from_base_18(pool.getDenormalizedWeight(ocean.OCEAN_address))
        dt_weight = from_base_18(pool.getDenormalizedWeight(dt_address))

        ocn_add_remove_list, dt_add_remove_list = ocean.pool.get_liquidity_history(poolAddress)
        ocn_add_remove_list = [(v, int(t)) for v, t in ocn_add_remove_list]
        dt_add_remove_list = [(v, int(t)) for v, t in dt_add_remove_list]

        ocn_reserve_history, dt_reserve_history, price_history = build_liquidity_and_price_history(
            ocn_add_remove_list, dt_add_remove_list, ocn_weight, dt_weight, swap_fee
        )

        result['oceanAddRemove'] = ocn_add_remove_list
        result['datatokenAddRemove'] = dt_add_remove_list
        result['oceanReserveHistory'] = ocn_reserve_history
        result['datatokenReserveHistory'] = dt_reserve_history
        result['datatokenPriceHistory'] = price_history
        return Response(json.dumps(result), 200, content_type='application/json')
    except Exception as e:
        logger.error(f'pools/history/{poolAddress}: {str(e)}', exc_info=1)
        return f'Get pool liquidity/price history failed: {str(e)}', 500


@pools.route('/liquidity/<poolAddress>', methods=['GET'])
def get_current_liquidity_stats(poolAddress):
    """

    :param poolAddress:
    :return:
    """
    try:
        data = get_request_data(request) or {}
        dt_address = data.get('datatokenAddress', None)
        from_block = data.get('fromBlock', None)
        to_block = data.get('toBlock', None)
        ocean = Ocean(ConfigProvider.get_config())
        pool_info = ocean.pool.get_short_pool_info(poolAddress, dt_address, from_block, to_block)
        return Response(json.dumps(pool_info), 200, content_type='application/json')
    except Exception as e:
        logger.error(f'pools/liquidity/{poolAddress}: {str(e)}')
        return f'Get pool current liquidity stats failed: {str(e)}', 500


@pools.route('/user/<userAddress>', methods=['GET'])
def get_user_balances(userAddress):
    """

    :param userAddress:
    :return:
    """
    try:
        data = get_request_data(request) or {}
        from_block = data.get('fromBlock', int(os.getenv('BFACTORY_BLOCK', 0)))
        ocean = Ocean(ConfigProvider.get_config())
        result = ocean.pool.get_user_balances(userAddress, from_block)
        return Response(json.dumps(result), 200, content_type='application/json')
    except Exception as e:
        logger.error(f'pools/user/{userAddress}: {str(e)}')
        return f'Get pool user balances failed: {str(e)}', 500




json_dict_no_metadata = {"publisherId": "0x2"}
json_dict_no_valid_metadata = {"publisherId": "0x4",
                               "main": {},
                               "assetId": "002"
                               }





# build accumulated liquidity
def get_accumulative_values(values_list):
    acc_values = [values_list[0]]
    n = 0
    for k, (v, t) in enumerate(values_list[1:]):
        if acc_values[n][1] == t:
            acc_values[n] = (acc_values[n][0] + v, t)
        else:
            acc_values.append((acc_values[n][0] + v, t))
            n += 1
    return acc_values


def build_liquidity_and_price_history(ocn_liquidity_changes, dt_liquidity_changes, ocn_weight, dt_weight, swap_fee):
    # numer = ov / ocn_weight
    # denom = dtv / dt_weight
    # ratio = numer / denom
    scale = 1.0 / (1.0 - swap_fee)
    # # price = ratio * scale
    weight_ratio = dt_weight / ocn_weight
    tot_ratio = weight_ratio * scale
    # p = ((ov / ocn_weight) / (dtv / dt_weight)) * (1.0 / (1.0 - swap_fee))
    # uint numer = bdiv(tokenBalanceIn, tokenWeightIn);
    # uint denom = bdiv(tokenBalanceOut, tokenWeightOut);
    # uint ratio = bdiv(numer, denom);
    # uint scale = bdiv(BONE, bsub(BONE, swapFee));
    # return  (spotPrice = bmul(ratio, scale));

    accumulated_ocn_values = get_accumulative_values(ocn_liquidity_changes)
    accumulated_dt_values = get_accumulative_values(dt_liquidity_changes)

    _ocn_values = []
    _dt_values = []
    prices = []
    all_times = sorted({tup[1] for tup in (accumulated_dt_values + accumulated_ocn_values)})

    i = 0
    j = 0
    ocnv, ocnt = accumulated_ocn_values[i]
    dtv, dtt = accumulated_dt_values[j]
    ocn_l = len(accumulated_ocn_values)
    dt_l = len(accumulated_dt_values)
    assert ocnt == dtt, 'The first timestamp does not match between ocean and datatoken liquidity.'
    assert all_times[0] == ocnt, ''

    for t in all_times:
        if (i+1) < ocn_l:
            _v, _t = accumulated_ocn_values[i + 1]
            if _t <= t:
                i += 1
                ocnv = _v

        if (j+1) < dt_l:
            _v, _t = accumulated_dt_values[j + 1]
            if _t <= t:
                j += 1
                dtv = _v

        _ocn_values.append((ocnv, t))
        _dt_values.append((dtv, t))
        prices.append(((ocnv / dtv) * tot_ratio, t))

    return _ocn_values, _dt_values, prices



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

import logging

import elasticsearch
from oceandb_driver_interface import OceanDb
from oceandb_driver_interface.search_model import FullTextModel, QueryModel


class Dao(object):

    def __init__(self, oceandb=None, config_file=None):
        self.oceandb = oceandb or OceanDb(config_file).plugin

    def get_all_listed_assets(self):
        return list(self.oceandb.list())

    def get_all_assets(self):
        return [a for a in self.oceandb.list()]

    def get(self, asset_id):
        try:
            asset = self.oceandb.read(asset_id)
        except elasticsearch.exceptions.NotFoundError as e:
            logging.error(f'Dao.get -- asset with id {asset_id} was not found, original error was:{str(e)}')
            raise

        except Exception as e:
            logging.error(f'Dao.get: {str(e)}')
            raise

        if asset is None or not self.is_listed(asset['service']):
            return None

        return asset

    def register(self, record, asset_id):
        return self.oceandb.write(record, asset_id)

    def update(self, record, asset_id):
        return self.oceandb.update(record, asset_id)

    def delete(self, asset_id):
        return self.oceandb.delete(asset_id)

    def delete_all(self):
        if hasattr(self.oceandb, 'delete_all'):
            self.oceandb.delete_all()
        else:
            assets = self.oceandb.list()
            for asset in assets:
                try:
                    self.delete(asset['id'])
                except Exception as e:
                    logging.error(f'Dao.delete_all: {str(e)}')

    def query(self, query):
        query_list = []
        if isinstance(query, QueryModel):
            query_result, count = self.oceandb.query(query)
        elif isinstance(query, FullTextModel):
            query_result, count = self.oceandb.text_query(query)
        else:
            raise TypeError('Unrecognized `query` type %s' % type(query))

        for f in query_result:
            if 'service' in f:
                if self.is_listed(f['service']):
                    query_list.append(f)

        return query_list, count

    @staticmethod
    def is_listed(services):
        for service in services:
            if service['type'] == 'metadata':
                if 'curation' in service['attributes'] and 'isListed' in service['attributes']['curation']:
                    return service['attributes']['curation']['isListed']

    def run_es_query(self, query, sort, page, offset, text=None):
        """do an elasticsearch native query.

        The query is expected to be in the elasticsearch search format.

        :return: list of objects that match the query.
        """
        assert page >= 1, 'page value %s is invalid' % page

        logging.debug(f'elasticsearch::query::{query}, text {text}')
        if sort is not None:
            self.oceandb._mapping_to_sort(sort.keys())
            sort = self.oceandb._sort_object(sort)
        else:
            sort = [{"_id": "asc"}]

        if text:
            sort = [{"_score": "desc"}] + sort
            if isinstance(text, str):
                text = [text]

            text = [t.strip() for t in text]
            text = [t.replace('did:op:', '0x') for t in text if t]

        if not query:
            query = {'match_all': {}}

        body = {
            'sort': sort,
            'from': (page - 1) * offset,
            'size': offset,
            'query': query
        }
        logging.info(f'running query: {body}')
        page = self.oceandb.driver.es.search(
            index=self.oceandb.driver.db_index,
            body=body,
            q=text or None
        )

        object_list = []
        for x in page['hits']['hits']:
            object_list.append(x['_source'])

        return object_list, page['hits']['total']



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

import configparser
import os

from elasticsearch import Elasticsearch
from flask import jsonify
from flask_swagger import swagger
from flask_swagger_ui import get_swaggerui_blueprint
from ocean_lib.config import Config as OceanConfig
from ocean_lib.config_provider import ConfigProvider
from ocean_lib.web3_internal.contract_handler import ContractHandler
from ocean_lib.web3_internal.web3_provider import Web3Provider
from pymongo import MongoClient

from aquarius.app.assets import assets
from aquarius.app.pools import pools
from aquarius.config import Config
from aquarius.constants import BaseURLs, Metadata
from aquarius.events.util import get_artifacts_path, get_network_name
from aquarius.myapp import app
from aquarius.events.events_monitor import EventsMonitor

config = Config(filename=app.config['CONFIG_FILE'])
aquarius_url = config.aquarius_url


def get_version():
    conf = configparser.ConfigParser()
    conf.read('.bumpversion.cfg')
    return conf['bumpversion']['current_version']


@app.route("/")
def version():
    info = dict()
    info['software'] = Metadata.TITLE
    info['version'] = get_version()
    info['plugin'] = config.module
    return jsonify(info)


@app.route("/health")
def health():
    return get_status()


@app.route("/spec")
def spec():
    swag = swagger(app)
    swag['info']['version'] = get_version()
    swag['info']['title'] = Metadata.TITLE
    swag['info']['description'] = Metadata.DESCRIPTION + '`' + aquarius_url + '`.'
    swag['info']['connected'] = get_status()
    # swag['basePath'] = BaseURLs.BASE_AQUARIUS_URL
    return jsonify(swag)


# Call factory function to create our blueprint
swaggerui_blueprint = get_swaggerui_blueprint(
    BaseURLs.SWAGGER_URL,
    aquarius_url + '/spec',
    config={  # Swagger UI config overrides
        'app_name': "Test application"
    },
)

# Register blueprint at URL
app.register_blueprint(swaggerui_blueprint, url_prefix=BaseURLs.SWAGGER_URL)
app.register_blueprint(assets, url_prefix=BaseURLs.ASSETS_URL)
app.register_blueprint(pools, url_prefix=BaseURLs.POOLS_URL)


def get_status():
    if config.get('oceandb', 'module') == 'elasticsearch':
        if Elasticsearch(config.db_url).ping():
            return 'Elasticsearch connected', 200
        else:
            return 'Not connected to any database', 400
    elif config.get('oceandb', 'module') == 'mongodb':
        if MongoClient(config.db_url).get_database(config.get('oceandb', 'db.name')).command(
                'ping'):
            return 'Mongodb connected', 200
        else:
            return 'Not connected to any database', 400
    else:
        return 'Not connected to any database', 400


# Start events monitoring if required
if bool(int(os.environ.get('EVENTS_ALLOW', '0'))):
    _config = OceanConfig(app.config['CONFIG_FILE'])
    ConfigProvider.set_config(_config)
    from ocean_lib.ocean.util import get_web3_connection_provider

    rpc = os.environ.get('EVENTS_RPC', '')
    Web3Provider.init_web3(provider=get_web3_connection_provider(rpc))
    ContractHandler.set_artifacts_path(get_artifacts_path())
    if get_network_name().lower() == 'rinkeby':
        from web3.middleware import geth_poa_middleware
        Web3Provider.get_web3().middleware_stack.inject(geth_poa_middleware, layer=0)

    monitor = EventsMonitor(
        Web3Provider.get_web3(),
        app.config['CONFIG_FILE']
    )
    monitor.start_events_monitor()


if __name__ == '__main__':
    if isinstance(config.aquarius_url.split(':')[-1], int):
        app.run(host=config.aquarius_url.split(':')[1],
                port=config.aquarius_url.split(':')[-1])
    else:
        app.run()



import json
import os
import time
import uuid

from eth_utils import remove_0x_prefix, add_0x_prefix
from web3 import Web3
from eth_account import Account
from web3.datastructures import AttributeDict

from aquarius.events.util import get_metadata_contract, deploy_datatoken
from aquarius.events.constants import EVENT_METADATA_CREATED, EVENT_METADATA_UPDATED
from tests.ddos.ddo_event_sample import ddo_event_sample

rpc = os.environ.get('EVENTS_RPC', None)
WEB3_INSTANCE = Web3(Web3.HTTPProvider(rpc))


test_account1 = Account.privateKeyToAccount(os.environ.get('EVENTS_TESTS_PRIVATE_KEY', None))
test_account2 = Account.privateKeyToAccount(os.environ.get('EVENTS_TESTS_PRIVATE_KEY2', None))
test_account3 = Account.privateKeyToAccount(os.environ.get('EVENTS_TESTS_PRIVATE_KEY3', None))
ecies_account = Account.privateKeyToAccount(os.environ.get('EVENTS_ECIES_PRIVATE_KEY', None))


def get_web3():
    return WEB3_INSTANCE


def prepare_did(text):
    prefix = 'did:op:'
    if text.startswith(prefix):
        text = text[len(prefix):]
    return add_0x_prefix(text)


def new_did(dt_address):
    return f'did:op:{remove_0x_prefix(dt_address)}'


def new_ddo(account, web3, name, ddo=None):
    _ddo = ddo if ddo else ddo_event_sample.copy()
    if 'publicKey' not in _ddo or not _ddo['publicKey']:
        _ddo['publicKey'] = [{'owner': ''}]
    _ddo['publicKey'][0]['owner'] = account.address
    _ddo['random'] = str(uuid.uuid4())
    dt_address = deploy_datatoken(web3, account.privateKey, name, name, account.address)
    _ddo['id'] = new_did(dt_address)
    _ddo['dataToken'] = dt_address
    return AttributeDict(_ddo)


def get_ddo(client, base_ddo_url, did):
    rv = client.get(
        base_ddo_url + f'/{did}',
        content_type='application/json'
    )
    try:
        fetched_ddo = json.loads(rv.data.decode('utf-8'))
        return fetched_ddo
    except (Exception, ValueError) as e:
        print(f'Error fetching cached ddo {did}: {e}.'
              f'\nresponse data was: {rv.data}')
        return None


def get_event(event_name, block, did, timeout=45):
    did = prepare_did(did)
    start = time.time()
    f = getattr(get_metadata_contract(get_web3()).events, event_name)().createFilter(fromBlock=block)
    logs = []
    while not logs:
        logs = f.get_all_entries()
        if not logs:
            time.sleep(0.2)

        if time.time() - start > timeout:
            break

    assert logs, 'no events found {event_name}, block {block}.'
    print(f'done waiting for {event_name} event, got {len(logs)} logs, and datatokens: {[l.args.dataToken for l in logs]}')
    _log = None
    for log in logs:
        if log.args.dataToken == did:
            _log = log
            break
    assert _log, f'event log not found: {event_name}, {block}, {did}'
    return _log


def send_tx(fn_name, tx_args, account):
    get_web3().eth.defaultAccount = account.address
    txn_hash = getattr(get_metadata_contract(get_web3()).functions, fn_name)(*tx_args).transact()
    txn_receipt = get_web3().eth.waitForTransactionReceipt(txn_hash)
    return txn_receipt


def send_create_update_tx(name, did, flags, data, account):
    print(f'{name}DDO {did} with flags: {flags} from {account.address}')
    did = prepare_did(did)
    print('*****************************************************************************\r\n')
    print(did)
    print('*****************************************************************************\r\n')
    r = send_tx(name, (did, flags, data), account)
    event_name = EVENT_METADATA_CREATED if name == 'create' else EVENT_METADATA_UPDATED
    events = getattr(get_metadata_contract(get_web3()).events, event_name)().processReceipt(r)
    # print(f'got {event_name} logs: {events}')
    return r



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

import configparser
import logging
import os

from aquarius.constants import ConfigSections

DEFAULT_NAME_AQUARIUS_URL = 'http://localhost:5000'

NAME_AQUARIUS_URL = 'aquarius.url'
ALLOW_FREE_ASSETS_ONLY = 'allowFreeAssetsOnly'
MODULE = 'module'
DB_HOSTNAME = 'db.hostname'
DB_PORT = 'db.port'

environ_names = {
    NAME_AQUARIUS_URL: ['AQUARIUS_URL', 'Aquarius URL'],
}

config_defaults = {
    ConfigSections.RESOURCES: {
        NAME_AQUARIUS_URL: DEFAULT_NAME_AQUARIUS_URL,
    }
}


class Config(configparser.ConfigParser):

    def __init__(self, filename=None, **kwargs):
        configparser.ConfigParser.__init__(self)

        self.read_dict(config_defaults)
        self._section_name = ConfigSections.RESOURCES
        self._oceandb_name = ConfigSections.OCEANBD
        self._logger = kwargs.get('logger', logging.getLogger(__name__))
        self._logger.debug('Config: loading config file %s', filename)

        if filename:
            with open(filename) as fp:
                text = fp.read()
                self.read_string(text)
        else:
            if 'text' in kwargs:
                self.read_string(kwargs['text'])
        self._load_environ()

    def _load_environ(self):
        for option_name, environ_item in environ_names.items():
            value = os.environ.get(environ_item[0])
            if value is not None:
                self._logger.debug('Config: setting environ %s = %s', option_name, value)
                self.set(self._section_name, option_name, value)

    def set_arguments(self, items):
        for name, value in items.items():
            if value is not None:
                self._logger.debug('Config: setting argument %s = %s', name, value)

                self.set(self._section_name, name, value)

    @property
    def aquarius_url(self):
        return self.get(self._section_name, NAME_AQUARIUS_URL)
    
    @property
    def allow_free_assets_only(self):
        return self.get(self._section_name, ALLOW_FREE_ASSETS_ONLY)

    @property
    def db_url(self):
        return self.get(self._oceandb_name, DB_HOSTNAME) + ":" + self.get(self._oceandb_name,
                                                                          DB_PORT)

    @property
    def module(self):
        return self.get(self._oceandb_name, MODULE)

    # static methods

    @staticmethod
    def get_environ_help():
        result = []
        for option_name, environ_item in environ_names.items():
            # codacy fix
            assert option_name
            result.append("{:20}{:40}".format(environ_item[0], environ_item[1]))
        return "\n".join(result)
